/*
 Alert.m
 
 Bernie Wylde
 */

#import "Alert.h"

static NSMutableArray *sCocoaAlerts;

@implementation CocoaAlertDelegate

@synthesize boolVal;
@synthesize lastSuppressionButtonState;

#pragma mark - alert delegate methods
- (BOOL)alertShowHelp:(NSAlert *)alert {
    DialogEventSetBool( NO );
    CallUserDefinedOnDialogFunction( alertShowHelp, [alert tag], [alert tag], NULL );
    return DialogEventBool();
}

- (void)alertDidEnd:(NSAlert *)alert returnCode:(NSInteger)returnCode contextInfo:(void *)contextInfo {
    lastSuppressionButtonState = [[alert suppressionButton] state];
    CallUserDefinedOnDialogFunction( alertDidEnd, returnCode, [alert tag], NULL );
}

@end

CocoaAlertDelegate *CocoaAlertDelegateGet( void )
{
    static CocoaAlertDelegate *sCocoaAlertDelegate = nil;
    if ( !sCocoaAlertDelegate ) sCocoaAlertDelegate = [[CocoaAlertDelegate alloc] init];
    return sCocoaAlertDelegate;
}

NSAlert *CocoaAlertWithTag( NSInteger tag )
{
    if ( tag ) {
        if ( tag > 1000000 ) return (NSAlert *)tag;// it's an AlertRef
        
        if ( sCocoaAlerts ) {
            for ( NSAlert *alert in sCocoaAlerts ) {
                if ( [alert tag] == tag ) return alert;
            }
        }
    }
    return nil;
}

void CocoaAlertAdd( NSAlert *alert )
{
    if ( !sCocoaAlerts ) sCocoaAlerts = [[NSMutableArray alloc] initWithCapacity:0];
    [sCocoaAlerts addObject:alert];
}

void CocoaAlertRemove( NSAlert *alert )
{ [sCocoaAlerts removeObject:alert]; }


// -- custom --
void AlertRemove( NSInteger tag )
{
    NSAlert *alert = CocoaAlertWithTag( tag );
    if ( alert ) {
        [sCocoaAlerts removeObject:alert];
    }
}

void AlertButtonSetKeyEquivalent( NSInteger alertTag, NSUInteger btnIndex, CFStringRef key )
{
    NSAlert *alert = CocoaAlertWithTag( alertTag );
    if ( alert ) {
        NSArray *buttons = [alert buttons];
        if ( btnIndex < [buttons count] ) {
            NSButton *btn = [buttons objectAtIndex:btnIndex];
            [btn setKeyEquivalent:(NSString *)key];
        }
    }
}

void AlertButtonSetKeyEquivalentModifierMask( NSInteger alertTag, NSUInteger btnIndex, NSUInteger mask )
{
    NSAlert *alert = CocoaAlertWithTag( alertTag );
    if ( alert ) {
        NSArray *buttons = [alert buttons];
        if ( btnIndex < [buttons count] ) {
            NSButton *btn = [buttons objectAtIndex:btnIndex];
            [btn setKeyEquivalentModifierMask:mask];
        }
    }
}


@implementation NSAlert (Additions)
@dynamic tagNumber;
@dynamic sheetNumber;

- (NSInteger)tag {
    return [objc_getAssociatedObject(self, @selector(tagNumber)) integerValue];
}

- (void)setTag:(NSInteger)tag {
    objc_setAssociatedObject(self, @selector(tagNumber), [NSNumber numberWithInteger:tag], OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}

- (BOOL)sheet {
    return [objc_getAssociatedObject(self, @selector(sheetNumber)) boolValue];
}

- (void)setSheet:(BOOL)sheet {
    objc_setAssociatedObject(self, @selector(sheetNumber), [NSNumber numberWithBool:sheet], OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}
@end

NSAlert *AlertWithTag( NSInteger tag )
{ return CocoaAlertWithTag( tag ); }

BOOL AlertExists( NSInteger tag )
{
    if ( sCocoaAlerts ) {
        for ( NSAlert *alert in sCocoaAlerts ) {
            if ( [alert tag] == tag ) return YES;
        }
    }
    return NO;
}

// Configure
void AlertLayout( NSInteger tag )
{
    NSAlert *alert = CocoaAlertWithTag( tag );
    if ( alert ) [alert layout];
}

NSAlertStyle AlertStyle( NSInteger tag )
{
    NSAlert *alert = CocoaAlertWithTag( tag );
    if ( alert ) return [alert alertStyle];
    return 0;
}

void AlertSetStyle( NSInteger tag, NSAlertStyle style )
{
    NSAlert *alert = CocoaAlertWithTag( tag );
    if ( alert ) [alert setAlertStyle:style];
}

void AlertSetAccessoryView( NSInteger alertTag, NSInteger viewTag )
{
    NSAlert *alert = CocoaAlertWithTag( alertTag );
    if ( alert ) {
        NSView *view = CocoaViewWithTag( viewTag );
        if ( view ) {
            [alert setAccessoryView:view];
        }
    }
}

void AlertSetShowsHelp( NSInteger tag, Boolean flag )
{
    NSAlert *alert = CocoaAlertWithTag( tag );
    if ( alert ) {
        [alert setShowsHelp:flag];
    }
}

CFStringRef AlertHelpAnchor( NSInteger tag )
{
    NSAlert *alert = CocoaAlertWithTag( tag );
    if ( alert ) {
        return (CFStringRef)[alert helpAnchor];
    }
    return nil;
}

void AlertSetHelpAnchor( NSInteger tag, CFStringRef anchorName )
{
    NSAlert *alert = CocoaAlertWithTag( tag );
    if ( alert ) {
        [alert setHelpAnchor:(NSString *)anchorName];
    }
}

void AlertSetShowsSuppressionButton( NSInteger tag, Boolean flag )
{
    NSAlert *alert = CocoaAlertWithTag( tag );
    if ( alert ) {
        [alert setShowsSuppressionButton:flag];
    }
}

void AlertSuppressionButtonSetState( NSInteger tag, NSCellStateValue value )
{
    NSAlert *alert = CocoaAlertWithTag( tag );
    if ( alert ) {
        [[alert suppressionButton] setState:value];
    }
}

NSCellStateValue AlertSuppressionButtonState( NSInteger tag )
{
    CocoaAlertDelegate *delegate = CocoaAlertDelegateGet();
    if ( delegate ) return delegate.lastSuppressionButtonState;
    return NSOffState;
}

void AlertSuppressionButtonSetTitle( NSInteger tag, CFStringRef title )
{
    NSAlert *alert = CocoaAlertWithTag( tag );
    if ( alert ) {
        [[alert suppressionButton] setTitle:(NSString *)title];
    }
}

CFStringRef AlertInformativeText( NSInteger tag )
{
    NSAlert *alert = CocoaAlertWithTag( tag );
    if ( alert ) return (CFStringRef)[alert informativeText];
    return nil;
}

void AlertSetInformativeText( NSInteger tag, CFStringRef string )
{
    NSAlert *alert = CocoaAlertWithTag( tag );
    if ( alert ) [alert setInformativeText:(NSString *)string];
}

CFStringRef AlertMessageText( NSInteger tag )
{
    NSAlert *alert = CocoaAlertWithTag( tag );
    if ( alert ) return (CFStringRef)[alert messageText];
    return nil;
}

void AlertSetMessageText( NSInteger tag, CFStringRef string )
{
    NSAlert *alert = CocoaAlertWithTag( tag );
    if ( alert ) [alert setMessageText:(NSString *)string];
}

void AlertSetIcon( NSInteger tag, void *inImage )
{
    NSAlert *alert = CocoaAlertWithTag( tag );
    if ( alert ) {
        NSImage *image = FixImageValue( inImage );
        if ( image ) [alert setIcon:image];
    }
}

void AlertSetIconNamed( NSInteger tag, CFStringRef imageName )
{
    NSAlert *alert = CocoaAlertWithTag( tag );
    if ( alert ) {
        NSImage *image = CocoaImageResource( (NSString *)imageName );
        if ( image ) [alert setIcon:image];
    }
}

// Buttons
CFArrayRef AlertButtons( NSInteger tag )
{
    NSAlert *alert = CocoaAlertWithTag( tag );
    if ( alert ) return (CFArrayRef)[alert buttons];
    return nil;
}

void AlertAddButtonWithTitle( NSInteger tag, CFStringRef title )
{
    NSAlert *alert = CocoaAlertWithTag( tag );
    if ( alert ) [alert addButtonWithTitle:(NSString *)title];
}

// Window
CocoaWindowRef AlertWindow( NSInteger tag )
{
    NSAlert *alert = CocoaAlertWithTag( tag );
    if ( alert ) return [alert window];
    return nil;
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
void AlertClose( NSInteger tag )
{
    NSAlert *alert = CocoaAlertWithTag( tag );
    if ( alert ) {
        NSWindow *parentWindow = [[alert window] sheetParent];
        if ( parentWindow ) {
            [parentWindow endSheet:[alert window]];
            [sCocoaAlerts removeObject:alert];
        }
    }
}
#endif

NSInteger CocoaAlert( NSInteger inTag, NSInteger inStyle, CFStringRef inMsg, CFStringRef inInfo, CFTypeRef inButtonTitles, NSInteger inSheetFlag )
{
    NSInteger response = 0;
    
    BOOL createdFlag = NO;
    
    BOOL show = ( inTag > 0 );
    inTag = ABS(inTag);
    
    NSString *msg = (NSString *)inMsg;
    NSString *info = (NSString *)inInfo;
    
    NSAlert *alert = CocoaAlertWithTag( inTag );
    
    if ( alert ) {    // exists
        
        // style
        if ( inStyle != kFBParamMissing ) {
            [alert setAlertStyle:(NSAlertStyle)inStyle];
        }
        
        if ( inSheetFlag != kFBParamMissing ) [alert setSheet:inSheetFlag];
        
    } else {        // create
        
        createdFlag = YES;
        
        // init
        alert = [[NSAlert alloc] init];
        
        // style
        if ( inStyle == kFBParamMissing ) {
            [alert setAlertStyle:NSWarningAlertStyle];
        } else {
            [alert setAlertStyle:(NSAlertStyle)inStyle];
        }
        
        // button titles
        if ( inButtonTitles ) {
            NSArray *titles = nil;
            
            if ( CFGetTypeID(inButtonTitles) == CFStringGetTypeID() ) {
                titles = [(NSString *)inButtonTitles componentsSeparatedByString:@";"];
                
            } else if ( CFGetTypeID(inButtonTitles) == CFArrayGetTypeID() ) {
                titles = (NSArray *)inButtonTitles;
            }
            
            if ( titles ) {
                for ( NSString *title in titles ) {
                    [alert addButtonWithTitle:title];
                }
            }
        }
        
        // sheet
        if ( inSheetFlag != kFBParamMissing ) {
            [alert setSheet:inSheetFlag];
        }
        
        // delegate
        [alert setDelegate:CocoaAlertDelegateGet()];
        
        // tag
        [alert setTag:inTag];
        
        // add to sheets array
        CocoaAlertAdd( alert );
    }
    
    // msg
    if ( msg != nil ) [alert setMessageText:msg];
    
    // info
    if ( info != nil ) [alert setInformativeText:info];
    
    // get delegate
    CocoaAlertDelegate *delegate = CocoaAlertDelegateGet();

    if ( show ) {
        // get front window
        NSInteger wndTag = 0;
        NSWindow *window = CocoaTargetWindow( &wndTag );
        
        if ( [alert sheet] && window ) {  // sheet
            
            // beginSheet
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
            [alert beginSheetModalForWindow:window completionHandler:^(NSModalResponse returnCode) {
                [delegate alertDidEnd:alert returnCode:returnCode contextInfo:nil];
                CocoaAlertRemove( alert );
            }];
#else
            [alert beginSheetModalForWindow:window modalDelegate:delegate didEndSelector:@selector(alertDidEnd:returnCode:contextInfo:) contextInfo:nil];
            CocoaAlertRemove( alert );
#endif
            
        } else {
            response = [alert runModal];
            delegate.lastSuppressionButtonState = [[alert suppressionButton] state];
            NSInteger tag = [alert tag];
            CocoaAlertRemove( alert );
            CallUserDefinedOnDialogFunction( alertDidEnd, response, tag, NULL );
        }
        
    } else {    // close
        if ( !createdFlag ) {
            delegate.lastSuppressionButtonState = [[alert suppressionButton] state];
            NSInteger tag = [alert tag];
            [[alert window] close];
            CocoaAlertRemove( alert );
            CallUserDefinedOnDialogFunction( alertDidEnd, 0, tag, NULL );
        }
    }
    
    return response;
}


