/*
 Image.m

 Bernie Wylde
 */

#import "Image.h"

// Images by name
ImageRef ImageNamed( CFStringRef name )
{ return CocoaImageResource( (NSString *)name ); }

void ImageSetName( ImageRef ref, CFStringRef name )
{ [ref setName:(NSString *)name]; }

CFStringRef ImageName( ImageRef ref )
{ return (CFStringRef)[ref name]; }

// Dynamically drawn image
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1080
ImageRef ImageWithDrawingHandler( CGSize size, Boolean flipped, CocoaImageDrawingHandlerType handler, void *userData )
{
    NSImage *image = [NSImage imageWithSize:NSSizeFromCGSize(size) flipped:flipped drawingHandler:^BOOL(NSRect rect) {
        if ( handler ) return (*handler)( NSRectToCGRect(rect), userData );
        return NO;
    }];
    return image;
}
#endif

// Image from resource file
ImageRef ImageByReferencingURL( CFURLRef url )
{ return [[[NSImage alloc] initByReferencingURL:(NSURL *)url] autorelease]; }

ImageRef ImageWithContentsOfURL( CFURLRef url )
{ return [[[NSImage alloc] initWithContentsOfURL:(NSURL *)url] autorelease]; }

// create from existing data
ImageRef ImageWithData( CFDataRef dta )
{ return [[[NSImage alloc] initWithData:(NSData *)dta] autorelease]; }

ImageRef ImageWithDataIgnoringOrientation( CFDataRef dta )
{ return [[[NSImage alloc] initWithDataIgnoringOrientation:(NSData *)dta] autorelease]; }

ImageRef ImageWithCGImage( CGImageRef image, CGSize size )
{ return [[[NSImage alloc] initWithCGImage:image size:NSSizeFromCGSize(size)] autorelease]; }

ImageRef ImageWithPasteboard( CocoaPasteboardRef pb )
{ return [[[NSImage alloc] initWithPasteboard:pb] autorelease]; }

// Create empty image
ImageRef ImageWithSize( CGSize size )
{ return [[[NSImage alloc] initWithSize:NSSizeFromCGSize(size)] autorelease]; }

// Attributes
CGSize ImageSize( ImageRef ref )
{ return NSSizeToCGSize( [ref size] ); }

void ImageSetSize( ImageRef ref, CGSize size )
{ [ref setSize:NSSizeFromCGSize( size )]; }

void ImageSetTemplate( ImageRef ref, Boolean flag )
{ [ref setTemplate:flag]; }

Boolean ImageIsTemplate( ImageRef ref )
{ return [ref isTemplate]; }

// Determine supported image types
Boolean ImageCanInitWithPasteboard( CocoaPasteboardRef pb )
{ return [NSImage canInitWithPasteboard:pb]; }

CFArrayRef ImageTypes( void )
{ return (CFArrayRef)[NSImage imageTypes]; }

CFArrayRef ImageUnfilteredTypes( void )
{ return (CFArrayRef)[NSImage imageUnfilteredTypes]; }

// Image representation
void ImageAddRepresentation( ImageRef ref, ImageRepRef rep )
{ [ref addRepresentation:rep]; }

void ImageAddRepresentations( ImageRef ref, CFArrayRef reps )
{ [ref addRepresentations:(NSArray *)reps]; }

CFArrayRef ImageRepresentations( ImageRef ref )
{ return (CFArrayRef)[ref representations]; }

void ImageRemoveRepresentation( ImageRef ref, ImageRepRef rep )
{ [ref removeRepresentation:rep]; }

ImageRepRef ImageBestRepresentationForRect( ImageRef ref, CGRect r, GraphicsContextRef ctx, CFDictionaryRef hints )
{ return [ref bestRepresentationForRect:NSRectFromCGRect(r) context:ctx hints:(NSDictionary *)hints]; }

// Image representation selection criteria
Boolean ImagePrefersColorMatch( ImageRef ref )
{ return [ref prefersColorMatch]; }

void ImageSetPrefersColorMatch( ImageRef ref, Boolean flag )
{ [ref setPrefersColorMatch:flag]; }

Boolean ImageUsesEPSOnResolutionMismatch( ImageRef ref )
{ return [ref usesEPSOnResolutionMismatch]; }

void ImageSetUsesEPSOnResolutionMismatch( ImageRef ref, Boolean flag )
{ [ref setUsesEPSOnResolutionMismatch:flag]; }

Boolean ImageMatchesOnMultipleResolution( ImageRef ref )
{ return [ref matchesOnMultipleResolution]; }

void ImageSetMatchesOnMultipleResolution( ImageRef ref, Boolean flag )
{ [ref setMatchesOnMultipleResolution:flag]; }

// Drawing
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1090
void ImageDrawInRect( ImageRef ref, CGRect rect )
{ [ref drawInRect:NSRectFromCGRect( rect )]; }
#endif

void ImageDrawAtPoint( ImageRef ref, CGPoint pt, CGRect fromRect, NSCompositingOperation operation, CGFloat fraction )
{ [ref drawAtPoint:NSPointFromCGPoint( pt ) fromRect:NSRectFromCGRect( fromRect ) operation:operation fraction:fraction]; }

void ImageDrawInRectFromRect( ImageRef ref, CGRect rect, CGRect fromRect, NSCompositingOperation op, CGFloat fraction )
{ [ref drawInRect:NSRectFromCGRect( rect ) fromRect:NSRectFromCGRect(fromRect) operation:op fraction:fraction]; }

void ImageDrawInRectFromRectRespectFlipped( ImageRef ref, CGRect rect, CGRect fromRect, NSCompositingOperation op, CGFloat fraction, Boolean respectFlipped, CFDictionaryRef hints )
{ [ref drawInRect:NSRectFromCGRect( rect ) fromRect:NSRectFromCGRect(fromRect) operation:op fraction:fraction respectFlipped:respectFlipped hints:(NSDictionary *)hints]; }

Boolean ImageDrawRepresentationInRect( ImageRef ref, ImageRepRef rep, CGRect rect )
{ return [ref drawRepresentation:rep inRect:NSRectFromCGRect( rect )]; }

// Drawing options
Boolean ImageIsValid( ImageRef ref )
{ return [ref isValid]; }

ColorRef ImageBackgroundColor( ImageRef ref )
{ return [ref backgroundColor]; }

void ImageSetBackgroundColor( ImageRef ref, ColorRef col )
{ [ref setBackgroundColor:col]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
NSEdgeInsets ImageCapInsets( ImageRef ref )
{ return [ref capInsets]; }

void ImageSetCapInsets( ImageRef ref, NSEdgeInsets insets )
{ [ref setCapInsets:insets]; }

NSImageResizingMode ImageResizingMode( ImageRef ref )
{ return [ref resizingMode]; }

void ImageSetResizingMode( ImageRef ref, NSImageResizingMode mode )
{ [ref setResizingMode:mode]; }
#endif

// Focus
void ImageLockFocus( ImageRef ref )
{ [ref lockFocus]; }

void ImageLockFocusFlipped( ImageRef ref, Boolean flag )
{ [ref lockFocusFlipped:flag]; }

void ImageUnlockFocus( ImageRef ref )
{ [ref unlockFocus]; }

// Alignment
CGRect ImageAlignmentRect( ImageRef ref )
{ return NSRectToCGRect( [ref alignmentRect] ); }

void ImageSetAlignmentRect( ImageRef ref, CGRect rect )
{ [ref setAlignmentRect:NSRectFromCGRect( rect )]; }

// Caching options
NSImageCacheMode ImageCacheMode( ImageRef ref )
{ return [ref cacheMode]; }

void ImageSetCacheMode( ImageRef ref, NSImageCacheMode mode )
{ [ref setCacheMode:mode]; }

void ImageRecache( ImageRef ref )
{ [ref recache]; }

// TIFF
CFDataRef ImageTIFFRepresentation( ImageRef ref )
{ return (CFDataRef)[ref TIFFRepresentation]; }

CFDataRef ImageTIFFRepresentationUsingCompression( ImageRef ref, NSTIFFCompression compression, float factor )
{ return (CFDataRef)[ref TIFFRepresentationUsingCompression:compression factor:factor]; }

// CGImage
CGImageRef ImageCGImageForProposedRect( ImageRef ref, CGRect *rect, GraphicsContextRef context, CFDictionaryRef hints )
{ return [ref CGImageForProposedRect:(NSRect *)rect context:context hints:(NSDictionary *)hints]; }

// Incremental loads
void ImageCancelIncrementalLoad( ImageRef ref )
{ [ref cancelIncrementalLoad]; }

// Hit testing
Boolean ImageHitTestRect( ImageRef ref, CGRect testRectDestSpace, CGRect imageRectDestSpace, GraphicsContextRef ctx, CFDictionaryRef hints, Boolean flipped )
{ return [ref hitTestRect:NSRectFromCGRect(testRectDestSpace) withImageDestinationRect:NSRectFromCGRect(imageRectDestSpace) context:ctx hints:(NSDictionary *)hints flipped:flipped]; }

// Image accessibility
CFStringRef ImageAccessibilityDescription( ImageRef ref )
{ return (CFStringRef)[ref accessibilityDescription]; }

void ImageSetAccessibilityDescription( ImageRef ref, CFStringRef description )
{ [ref setAccessibilityDescription:(NSString *)description]; }

// Core animation
CFTypeRef ImageLayerContentsForContentsScale( ImageRef ref, CGFloat scale )
{ return (CFTypeRef)[ref layerContentsForContentsScale:scale]; }

CGFloat ImageRecommendedLayerContentsScale( ImageRef ref, CGFloat preferredScale )
{ return [ref recommendedLayerContentsScale:preferredScale]; }

// Axis matching
Boolean ImageMatchesOnlyOnBestFittingAxis( ImageRef ref )
{ return [ref matchesOnlyOnBestFittingAxis]; }

void ImageSetMatchesOnlyOnBestFittingAxis( ImageRef ref, Boolean flag )
{ [ref setMatchesOnlyOnBestFittingAxis:flag]; }

