/*
 Workspace.m

 Bernie Wylde
 */

#import "Workspace.h"

static NSMutableArray *sCocoaWorkspaceNotificationCenters;

@implementation CocoaWorkspaceNotificationCenter : NSObject
@synthesize callback;
@synthesize name;

- (id)initWithCallback:(CocoaWorkspaceNotificationCenterCallbackType)cb name:(NSString *)nm {
    if ( self = [super init] ) {
        self.callback = cb;
        self.name = nm;
    }
    return self;
}

- (void)didReceiveNotification:(NSNotification *)notification {
    if ( callback ) (*callback)(notification);
}
@end

// -- private --
NSMutableArray *CocoaWorkspaceNotificationCenters( void )
{
    if ( !sCocoaWorkspaceNotificationCenters ) sCocoaWorkspaceNotificationCenters = [[NSMutableArray alloc] initWithCapacity:0];
    return sCocoaWorkspaceNotificationCenters;
}

// -- public --
// Shared
WorkspaceRef WorkspaceShared( void )
{ return [NSWorkspace sharedWorkspace]; }

// Notification center
NotificationCenterRef WorkspaceNotificationCenter( void )
{ return [[NSWorkspace sharedWorkspace] notificationCenter]; }

void WorkspaceNotificationCenterAddObserver( void *callback, CFStringRef name, CFTypeRef object )
{
    NSMutableArray *centers = CocoaWorkspaceNotificationCenters();
    CocoaWorkspaceNotificationCenter *center = [[CocoaWorkspaceNotificationCenter alloc] initWithCallback:callback name:(NSString *)name];
    [centers addObject:center];
    [[[NSWorkspace sharedWorkspace] notificationCenter] addObserver:center selector:@selector(didReceiveNotification:) name:(NSString *)name object:(id)object];
}

void WorkspaceNotificationCenterRemoveObserver( void *callback, CFStringRef name )
{
    NSMutableArray *centers = CocoaWorkspaceNotificationCenters();
    for ( CocoaWorkspaceNotificationCenter *center in centers ) {
        if ( (center.callback == callback) && ([[center name] isEqualToString:(NSString *)name]) ) {
            [[[NSWorkspace sharedWorkspace] notificationCenter] removeObserver:center name:(NSString *)name object:nil];
            [centers removeObject:center];
            break;
        }
    }
}

void WorkspaceNotificationCenterPostNotification( NotificationRef note )
{ [[[NSWorkspace sharedWorkspace] notificationCenter] postNotification:note]; }

void WorkspaceNotificationCenterPostNotificationName( CFStringRef name, CFTypeRef object, CFDictionaryRef userInfo )
{ [[[NSWorkspace sharedWorkspace] notificationCenter] postNotificationName:(NSString *)name object:object userInfo:(NSDictionary *)userInfo]; }

// Opening files
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
RunningApplicationRef WorkspaceOpenURLWithOptions( CFURLRef url, NSWorkspaceLaunchOptions options, CFDictionaryRef configuration, ErrorRef *err )
{ return [[NSWorkspace sharedWorkspace] openURL:(NSURL *)url options:options configuration:(NSDictionary *)configuration error:err]; }
#endif

Boolean WorkspaceOpenURL( CFURLRef url )
{ return [[NSWorkspace sharedWorkspace] openURL:(NSURL *)url]; }

Boolean WorkspaceOpenURLWithApplication( CFURLRef url, CFStringRef appName, Boolean deactivate )
{ return [[NSWorkspace sharedWorkspace] openFile:[(NSURL *)url path] withApplication:(NSString *)appName andDeactivate:deactivate]; }

Boolean WorkspaceOpenURLsWithApplicationAtURL( CFArrayRef urls, CFURLRef appURL, NSWorkspaceLaunchOptions options, CFDictionaryRef configuration )
{
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
    return ( [[NSWorkspace sharedWorkspace] openURLs:(NSArray *)urls withApplicationAtURL:(NSURL *)appURL options:options configuration:(NSDictionary *)configuration error:nil] != nil );
#endif
    return false;
}

Boolean WorkspaceOpenURLWithApplicationAtURL( CFURLRef url, CFURLRef appURL, NSWorkspaceLaunchOptions options, CFDictionaryRef configuration )
{
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
    return ( [[NSWorkspace sharedWorkspace] openURLs:[NSArray arrayWithObject:(NSURL *)url] withApplicationAtURL:(NSURL *)appURL options:options configuration:(NSDictionary *)configuration error:nil] != nil );
#endif
    return false;
}

Boolean WorkspaceLaunchApplication( CFStringRef appName )
{ return [[NSWorkspace sharedWorkspace] launchApplication:(NSString *)appName]; }

RunningApplicationRef WorkspaceLaunchApplicationAtURL( CFURLRef url, NSWorkspaceLaunchOptions options, CFDictionaryRef configuration, ErrorRef *err )
{ return [[NSWorkspace sharedWorkspace] launchApplicationAtURL:(NSURL *)url options:options configuration:(NSDictionary *)configuration error:err]; }

void WorkspaceHideOtherApplications( void )
{ [[NSWorkspace sharedWorkspace] hideOtherApplications]; }

void WorkspaceDuplicateURLs( CFArrayRef urls )
{ [[NSWorkspace sharedWorkspace] duplicateURLs:(NSArray *)urls completionHandler:nil]; }

void WorkspaceRecycleURL( CFURLRef url)
{ [[NSWorkspace sharedWorkspace] recycleURLs:[NSArray arrayWithObject:(NSURL *)url] completionHandler:nil]; }

void WorkspaceRecycleURLs( CFArrayRef urls )
{ [[NSWorkspace sharedWorkspace] recycleURLs:(NSArray *)urls completionHandler:nil]; }

void WorkspaceActivateFileViewerSelectingURLs( CFArrayRef fileURLs )
{ [[NSWorkspace sharedWorkspace] activateFileViewerSelectingURLs:(NSArray *)fileURLs]; }

// UTI
CFStringRef WorkspaceTypeOfFileAtURL( CFURLRef url )
{ return (CFStringRef)[[NSWorkspace sharedWorkspace] typeOfFile:[(NSURL *)url path] error:nil]; }

CFStringRef WorkspaceLocalizedDescriptionForType( CFStringRef type )
{ return (CFStringRef)[[NSWorkspace sharedWorkspace] localizedDescriptionForType:(NSString *)type]; }

CFStringRef WorkspacePreferredFilenameExtensionForType( CFStringRef typeName )
{ return (CFStringRef)[[NSWorkspace sharedWorkspace] preferredFilenameExtensionForType:(NSString *)typeName]; }

Boolean WorkspaceFilenameExtensionIsValidForType( CFStringRef filenameExtension, CFStringRef typeName )
{ return [[NSWorkspace sharedWorkspace] filenameExtension:(NSString *)filenameExtension isValidForType:(NSString *)typeName]; }

Boolean WorkspaceFileTypeConformsToType( CFStringRef firstTypeName, CFStringRef secondTypeName )
{ return [[NSWorkspace sharedWorkspace] type:(NSString *)firstTypeName conformsToType:(NSString *)secondTypeName]; }

Boolean WorkspaceGetInfoForFileAtURL( CFURLRef url, CFStringRef *appName, CFStringRef *type )
{ return [[NSWorkspace sharedWorkspace] getInfoForFile:[(NSURL *)url path] application:(NSString **)appName type:(NSString **)type]; }

CFURLRef WorkspaceURLForApplicationToOpenURL( CFURLRef url )
{ return (CFURLRef)[[NSWorkspace sharedWorkspace] URLForApplicationToOpenURL:(NSURL *)url]; }

Boolean WorkspaceGetFileSystemInfoForURL( CFURLRef url, Boolean *isRemovable, Boolean *isWritable, Boolean *isUnmountable, CFStringRef *description, CFStringRef *type )
{ return [[NSWorkspace sharedWorkspace] getFileSystemInfoForPath:[(NSURL *)url path] isRemovable:(BOOL *)isRemovable isWritable:(BOOL *)isWritable isUnmountable:(BOOL *)isUnmountable description:(NSString **)description type:(NSString **)type]; }

Boolean WorkspaceIsFilePackageAtURL( CFURLRef url )
{ return [[NSWorkspace sharedWorkspace] isFilePackageAtPath:[(NSURL *)url path]]; }

RunningApplicationRef WorkspaceFrontmostApplication( void )
{ return [[NSWorkspace sharedWorkspace] frontmostApplication]; }

CFArrayRef WorkspaceRunningApplications( void )
{ return (CFArrayRef)[[NSWorkspace sharedWorkspace] runningApplications]; }

RunningApplicationRef WorkspaceMenuBarOwningApplication( void )
{ return [[NSWorkspace sharedWorkspace] menuBarOwningApplication]; }

ImageRef WorkspaceIconForFileAtURL( CFURLRef url )
{ return [[NSWorkspace sharedWorkspace] iconForFile:[(NSURL *)url path]]; }

ImageRef WorkspaceIconForFileType( CFStringRef type )
{ return [[NSWorkspace sharedWorkspace] iconForFileType:(NSString *)type]; }

void WorkspaceSetIconForFileAtURL( ImageRef icon, CFURLRef url, NSWorkspaceIconCreationOptions options )
{ [[NSWorkspace sharedWorkspace] setIcon:icon forFile:[(NSURL *)url path] options:options]; }

// Unmount
Boolean WorkspaceUnmountAndEjectDeviceAtURL( CFURLRef url, ErrorRef *err )
{ return [[NSWorkspace sharedWorkspace] unmountAndEjectDeviceAtURL:(NSURL *)url error:err]; }

Boolean WorkspaceShowSearchResultsForQueryString( CFStringRef string )
{ return [[NSWorkspace sharedWorkspace] showSearchResultsForQueryString:(NSString *)string]; }

CFArrayRef WorkspaceFileLabelColors( void )
{ return (CFArrayRef)[[NSWorkspace sharedWorkspace] fileLabelColors]; }

CFArrayRef WorkspaceFileLabels( void )
{ return (CFArrayRef)[[NSWorkspace sharedWorkspace] fileLabels]; }


CFURLRef WorkspaceURLForApplicationWithBundleIdentifier( CFStringRef identifier )
{ return (CFURLRef)[[NSWorkspace sharedWorkspace] URLForApplicationWithBundleIdentifier:(NSString *)identifier]; }

CFURLRef WorkspaceURLForApplication( CFStringRef name )
{ return (CFURLRef)[NSURL fileURLWithPath:[[NSWorkspace sharedWorkspace] fullPathForApplication:(NSString *)name]]; }

void WorkspaceNoteFileSystemChangedAtURL( CFURLRef url )
{ [[NSWorkspace sharedWorkspace] noteFileSystemChanged:[(NSURL *)url path]]; }


// - desktop image -
CFURLRef WorkspaceDesktopImageURLForScreen( ScreenRef screen )
{ return (CFURLRef)[[NSWorkspace sharedWorkspace] desktopImageURLForScreen:screen]; }

void WorkspaceSetDesktopImageURLForScreen( CFURLRef url, ScreenRef screen, CFDictionaryRef options, ErrorRef *err )
{ [[NSWorkspace sharedWorkspace] setDesktopImageURL:(NSURL *)url forScreen:screen options:(NSDictionary *)options error:err]; }

CFDictionaryRef WorkspaceDesktopImageOptionsForScreen( ScreenRef screen )
{ return (CFDictionaryRef)[[NSWorkspace sharedWorkspace] desktopImageOptionsForScreen:screen]; }

// Instance methods
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400 && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101400
void WorkspaceRequestAuthorizationOfType( NSWorkspaceAuthorizationType type, WorkspaceRequestAuthorizationCompletionHandlerType completionHandler, void *userData )
{
    [[NSWorkspace sharedWorkspace] requestAuthorizationOfType:type completionHandler:^(NSWorkspaceAuthorization *authorization, NSError *error) {
        if ( completionHandler ) (*completionHandler)(authorization,error,userData);
    }];
}
#endif
